/*
 * IntonacjaPanel.java
 *
 * Created on 22 stycze 2005, 14:13
 */

package jsampa;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Toolkit;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.NumberFormat;
import java.util.Properties;
import java.util.Vector;

/**
 *
 * @author  michalus
 */
public class IntonacjaPanel extends javax.swing.JPanel {
    
    /** Najwieksza wartosc na osi y*/
    private int maxWartosc = 300;
    /** Margines panelu */
    private int marginesPoziomy = 25;
    /** Margines panelu */
    private int marginesPionowy = 25;
    /** Formatowanie liczb na panelu */
    private NumberFormat nf;
    
    private Vector punkty;
    
    private Punkt wyrysowywanyPunkt;
    
    private String nazwa;
    
    private int dokladnoscPodzialki = 15;
    
    /** Creates new form IntonacjaPanel */
    public IntonacjaPanel(String nazwa) {
        
        this.nazwa = nazwa;
        
        initComponents();
        
        setBackground(Color.WHITE);
        
        nf = NumberFormat.getInstance();
        nf.setMaximumFractionDigits(3);
        nf.setMinimumFractionDigits(0);
        
        punkty = wgrajUstawienia();
        wyrysowywanyPunkt = null;
        
        repaint();
    }
    
    public void nagrajUstawienia() {
        Properties ustawienia = new Properties();
        
        try {            
            
            ustawienia.setProperty("ilosc", ""+punkty.size());
            
            for(int i=0; i<punkty.size(); i++) {
                Punkt p = (Punkt)punkty.get(i);
                ustawienia.setProperty(("p"+i+"_x"), ""+p.getX());
                ustawienia.setProperty(("p"+i+"_y"), ""+p.getY());
            }
            
//            FileOutputStream out = new FileOutputStream(nazwa);
//            ustawienia.store(out, "Ustawienia intonacji");
//            out.close();
            
            FileOutputStream out = new FileOutputStream(nazwa+".xml");
            ustawienia.storeToXML(out, "Ustawienia intonacji");
            out.close();
            
        } catch(IOException e) {
            System.out.println("Nie mozna bylo zapisac ustawien programu!");
            e.printStackTrace();
        }
    }
    
    public Vector wgrajUstawienia() {
        
        Properties ustawienia = new Properties();
        Vector nowy = new Vector();
        
        try {
            FileInputStream in = new FileInputStream(nazwa+".xml"); //wczytanie zapisanych ustawien
            ustawienia.loadFromXML(in);
            in.close();
            
            int ilosc = Integer.parseInt(ustawienia.getProperty("ilosc", "0"));
            for(int i=0; i<ilosc; i++) {
                double x = Double.parseDouble(ustawienia.getProperty(("p"+i+"_x"), "0"));
                double y = Double.parseDouble(ustawienia.getProperty(("p"+i+"_y"), "0"));
                Punkt p = new Punkt(x, y);
                nowy.add(p);
            }
            
        } catch(IOException e) {            
            System.out.println("Ustawienia nie byy wczeniej zapisywane.");
            //e.printStackTrace();
        } catch(NumberFormatException e) {
            //e.printStackTrace();
        }    
        
        if(nowy.size()==0) {
            nowy.add(new Punkt(0, 200));
            nowy.add(new Punkt(1, 220));
        }
        
        return nowy;
    }
    
    private void sortujPunkty() {        
        
        for(int i=0; i<punkty.size(); i++) {
            for(int j=i+1; j<punkty.size(); j++) {
                Punkt p1 = (Punkt)punkty.get(i);
                Punkt p2 = (Punkt)punkty.get(j);
                
                if(p1.getX()>p2.getX()) {
                    punkty.remove(p1);
                    punkty.remove(p2);
                    punkty.add(i, p2);
                    punkty.add(j, p1);
                }
            }
        }
        
        nagrajUstawienia();
    }
    
    /** Zwraca punkty wedlug ktorej tworzona jest intonacja
     */
    public Vector getObwiednia() {
        return this.punkty;
    }
    
    public void paint(Graphics g) {
        super.paint(g);
        
        //wyrysowanie magrinesu
        g.setColor(Color.LIGHT_GRAY);        
        g.drawRect(marginesPoziomy, marginesPionowy, getWidth()-2*marginesPoziomy, getHeight()-2*marginesPionowy);
        
        double szerokosc = getWidth() - marginesPoziomy*2;
        double wysokosc = (double)(getHeight() - marginesPionowy*2)/maxWartosc;
        
        //rysowanie podzialki        
        for(int i=0; i<=dokladnoscPodzialki; i++) {
            
            
            
            int wys = (int)(Math.round((double)(getHeight()-marginesPionowy*2)/dokladnoscPodzialki)*i);
            int wartosc = (int)(wys/wysokosc);
            
            g.setColor(Color.LIGHT_GRAY);
            g.drawLine(
                    (int)Math.round((double)marginesPoziomy/4), 
                    getHeight()-marginesPionowy-wys-1, 
                    (int)Math.round((double)marginesPoziomy/2), 
                    getHeight()-marginesPionowy-wys-1);
            
            g.setColor(Color.GRAY);
            g.drawString(
                    ""+wartosc, 
                    2, 
                    getHeight()-marginesPionowy-wys-2);
        } 
        
        //wyrysowanie punktow
        
        
        if(punkty.size()>0) {
            g.setColor(Color.RED);
            Punkt p = (Punkt)punkty.get(0);
            g.fillOval((int)(marginesPoziomy+p.getX()*szerokosc)-4, (int)((getHeight()-marginesPionowy-p.getY()*wysokosc))-4, 9, 9);
            g.setColor(Color.LIGHT_GRAY);
            g.drawString(
                    ""+nf.format(p.getX()*100)+"%",
                    (int)(marginesPoziomy+p.getX()*szerokosc)-20, 
                    (int)(getHeight()-(marginesPionowy+p.getY()*wysokosc))-20);
        }
        
        for(int i=1; i<punkty.size(); i++) {
            g.setColor(Color.RED);
            Punkt p = (Punkt)punkty.get(i);
            Punkt p0 = (Punkt)punkty.get(i-1);
            g.fillOval((int)(marginesPoziomy+p.getX()*szerokosc)-4, (int)((getHeight()-marginesPionowy-p.getY()*wysokosc))-4, 9, 9);
            g.drawLine(
                    (int)(marginesPoziomy+p.getX()*szerokosc), 
                    (int)(getHeight()-(marginesPionowy+p.getY()*wysokosc)),
                    (int)(marginesPoziomy+p0.getX()*szerokosc), 
                    (int)(getHeight()-(marginesPionowy+p0.getY()*wysokosc)));
            g.setColor(Color.LIGHT_GRAY);
            g.drawString(
                    ""+nf.format(p.getX()*100)+"%",
                    (int)(marginesPoziomy+p.getX()*szerokosc)-20, 
                    (int)(getHeight()-(marginesPionowy+p.getY()*wysokosc))-20);
        }
        
        if(wyrysowywanyPunkt!=null) {
            g.setColor(Color.GREEN);            
            g.fillOval((int)(marginesPoziomy+wyrysowywanyPunkt.getX()*szerokosc)-4, (int)(getHeight()-marginesPionowy-wyrysowywanyPunkt.getY()*wysokosc)-4, 9, 9);
        }
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents

        setLayout(null);

        addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                formMouseClicked(evt);
            }
        });
        addMouseMotionListener(new java.awt.event.MouseMotionAdapter() {
            public void mouseMoved(java.awt.event.MouseEvent evt) {
                formMouseMoved(evt);
            }
        });

    }//GEN-END:initComponents

    private void formMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMouseClicked
        
        int x = evt.getX() - marginesPoziomy;
        int y = this.getHeight()- evt.getY() - marginesPionowy;
        
        double vx = (double)x/(this.getWidth()-marginesPoziomy*2);
        double vy = (double)y/(this.getHeight()-marginesPionowy*2)*maxWartosc;     
        
        if(wyrysowywanyPunkt==null) {
            if(evt.getButton()==evt.BUTTON1) {
                Punkt p = new Punkt(vx, vy);
                punkty.add(p);
                sortujPunkty();
                repaint();
            } else {
                //--
            }
        } else {
            if(evt.getButton()==evt.BUTTON1) {
                punkty.remove(wyrysowywanyPunkt);
                Punkt p = new Punkt(wyrysowywanyPunkt.getX(), vy);
                wyrysowywanyPunkt = p;
                punkty.add(p);
                sortujPunkty();
                repaint();
            } else {
                
                if(wyrysowywanyPunkt.getX()!=0 && wyrysowywanyPunkt.getX()!=1) {    //nie mozna usunac pierwszego i ostatniego punktu
                    punkty.remove(wyrysowywanyPunkt);
                    sortujPunkty();
                    repaint();
                } else {
                    Toolkit.getDefaultToolkit().beep();
                }
                
            }
        }
    }//GEN-LAST:event_formMouseClicked

    private void formMouseMoved(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMouseMoved
        
        
        int x = evt.getX() - marginesPoziomy;
        int y = this.getHeight()- evt.getY() - marginesPionowy;
        
        double vx = (double)x/(this.getWidth()-marginesPoziomy*2);
        double vy = (double)y/(this.getHeight()-marginesPionowy*2)*maxWartosc;       
        boolean zmianaPunktu = false;
        
        if(wyrysowywanyPunkt!=null) zmianaPunktu = true;
        wyrysowywanyPunkt = null;        
        
        for(int i=0; i<punkty.size(); i++) {
            Punkt p = (Punkt)punkty.get(i);
            
            if(Math.abs(p.getX()-vx)<0.005) {
                wyrysowywanyPunkt = p; 
                zmianaPunktu = true;
                break;
            }
        }
        
        String infop = "";
        
        if(wyrysowywanyPunkt!=null) {
            infop = " punkt: ("+nf.format(wyrysowywanyPunkt.getX())+"; "+nf.format(wyrysowywanyPunkt.getY())+";)";
        }
        
        if(vx>=0 && vx<=1 && vy>=0 && vy<=maxWartosc)
            this.setToolTipText("pozycja ~ ("+nf.format(vx)+"; "+nf.format(vy)+";)"+infop);
        else
            this.setToolTipText(null);
        
        if(zmianaPunktu)
            repaint();
        
    }//GEN-LAST:event_formMouseMoved
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
    
}
