/*
 * Transkrybowane.java
 *
 * Created on 11 stycze 2005, 23:04
 */

package jsampa;

import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.text.NumberFormat;
import java.util.Properties;
import java.util.Vector;
import javax.swing.JFrame;

/**
 *
 * @author michalus
 */
public class Transkrybowane {
    
    public Vector w;
    public Vector sylaby;
    
    public int[] czas;
    public int[] pitch;
    
    public static final int
            ZDANIE_PYTAJACE = 10,
            ZDANIE_OZNAJMUJACE = 11,
            ZDANIE_WYKRZYKNIKOWE = 12;
    
    public int
            czasTrwaniaDomyslny = -1,
            czasTrwaniaPlozywnych = -1,
            czasTrwaniaSzumiacych = -1,
            czasTrwaniaSamoglosek = -1,
            czasTrwaniaNosowych = -1,
            czasTrwaniaSpojnikow = -1;
    
    public int
            podstawowaPitch = -1;
    
    public String[]
            nosowe = null,
            plozywne = null,
            samogloski = null,
            spojniki = null,
            szumiace = null;
    
    private void domyslneCzasyTrwania() {
        
        Properties czasy = new Properties();
        
        czasy.setProperty("domyslny", "90");
        czasy.setProperty("nosowe", "80");
        czasy.setProperty("plozywne", "45");
        czasy.setProperty("samogloski", "55");
        czasy.setProperty("spojniki", "80");
        czasy.setProperty("szumiace", "80");
        czasy.setProperty("podstawa_pitch", "200");
        
        try {            
            FileOutputStream out = new FileOutputStream("czas_oraz_pitch.xml");
            czasy.storeToXML(out, "Czasy trwania oraz ustawienia wysokoci gosu");
            out.close();            
        } catch(IOException eio) {
            System.out.println("Nie mozna bylo zapisac ustawien programu!");
            eio.printStackTrace();
        }
    }     
    
    private void wczytajCzasyTrwan() {
        
        Properties czasy = new Properties();
        
        try {
            FileInputStream in = new FileInputStream("czas_oraz_pitch.xml"); //wczytanie zapisanych ustawien
            czasy.loadFromXML(in);
            in.close();
            
            czasTrwaniaDomyslny = Integer.parseInt(czasy.getProperty("domyslny", "80"));
            czasTrwaniaNosowych = Integer.parseInt(czasy.getProperty("nosowe", "70"));
            czasTrwaniaPlozywnych = Integer.parseInt(czasy.getProperty("plozywne", "35"));
            czasTrwaniaSamoglosek = Integer.parseInt(czasy.getProperty("samogloski", "45"));
            czasTrwaniaSpojnikow = Integer.parseInt(czasy.getProperty("spojniki", "70"));
            czasTrwaniaSzumiacych = Integer.parseInt(czasy.getProperty("szumiace", "70"));
            podstawowaPitch = Integer.parseInt(czasy.getProperty("podstawa_pitch", "200"));
            
        } catch(IOException e) {
            System.out.println("Ustawienia nie byy wczeniej zapisywane.");
            domyslneCzasyTrwania();
        } catch(NumberFormatException nfe) {
            System.out.println("Bdy w pliku konfiguracyjnym, plik zostaje naprawiony.");
            domyslneCzasyTrwania();
        }
        
    }
    
    /** Creates a new instance of Transkrybowane */
    public Transkrybowane(Vector w, MaskaIntonacji mi) {
        
        this.w = w;
        this.czas = new int[w.size()];
        this.pitch = new int[w.size()];
        
        // plozywne (zwarte)  b, p, d, t, g, k
        plozywne = new String[]{"b", "p", "d", "t", "g", "k"};
        
        // szumiace s, z, c, dz, sz, /rz, cz, d, , d
        szumiace = new String[]{"s", "z", "c", "dz", "S", "Z", "tS", "dZ'", "tS'", "dz'"};
        
        // nosowe m, n, , ng
        nosowe = new String[]{"m", "n'", "n"};
        
        samogloski = new String[]{"a", "e", "i", "o", "u", "I"};
        
        spojniki = new String[]{".", "?", "!", ","};
        
        wczytajCzasyTrwan();
        
        for(int i=0; i<czas.length; i++) {
            
            czas[i] = czasTrwaniaDomyslny;
            
            String t = (String)w.get(i);
            
            for(int j=0; j<plozywne.length; j++)
                if(t.equals(plozywne[j])) czas[i] = (int)Math.round(czasTrwaniaPlozywnych+Math.random()*10-5);
            
            for(int j=0; j<nosowe.length; j++)
                if(t.equals(nosowe[j])) czas[i] = (int)Math.round(czasTrwaniaNosowych+Math.random()*10-5);
            
            for(int j=0; j<samogloski.length; j++)
                if(t.equals(samogloski[j])) czas[i] = (int)Math.round(czasTrwaniaSamoglosek+Math.random()*10-5);
            
            for(int j=0; j<szumiace.length; j++)
                if(t.equals(szumiace[j])) czas[i] = (int)Math.round(czasTrwaniaSzumiacych+Math.random()*10-5);
            
            for(int j=0; j<spojniki.length; j++)
                if(t.equals(spojniki[j])) czas[i] = (int)Math.round(czasTrwaniaSpojnikow+Math.random()*10-5);
        }
        
        for(int i=0; i<czas.length; i++) {
            pitch[i] = podstawowaPitch;
        }
        
        //stosuje intonacje z ustalonych obwiedni
        
        for(int k=0; k<mi.getIloscPrzedzialow(); k++) {
            
            boolean
                 wykrzyknikowe = false,
                 pytajace = false,
                 oznajmujace = false;
            
            switch(mi.getIntonacja(k)) {
                case ZDANIE_WYKRZYKNIKOWE: wykrzyknikowe = true; break;
                case ZDANIE_PYTAJACE: pytajace = true; break;
                case ZDANIE_OZNAJMUJACE: oznajmujace = true; break;
                default:
                    oznajmujace = true;
            }
            
            // obwiednia dla zdan oznajmiajacych, pytajacych i wykrzyknikowych
            
            double
                    a = 0,
                    b = 0;
            
            Vector obwiednia = null;
            
            if(oznajmujace) {
                
                IntonacjaPanel ip = new IntonacjaPanel("intonacja_oznajmujaca");
                obwiednia = ip.getObwiednia();
            }
            
            if(pytajace) {
                
                IntonacjaPanel ip = new IntonacjaPanel("intonacja_pytajaca");
                obwiednia = ip.getObwiednia();
            }
            
            if(wykrzyknikowe) {
                
                IntonacjaPanel ip = new IntonacjaPanel("intonacja_wykrzyknikowa");
                obwiednia = ip.getObwiednia();
            }
            
            for(int i=mi.getPoczatek(k); i<mi.getKoniec(k); i++) {
                
                int n = 1;
                Punkt pb = (Punkt)obwiednia.get(n);
                
                NumberFormat nf = NumberFormat.getInstance();
                nf.setMaximumFractionDigits(5);
                
                while((double)(mi.getPoczatek(k)-i)/(double)(mi.getPoczatek(k)-mi.getKoniec(k))>pb.getX()) {
                    n++;
                    pb = (Punkt)obwiednia.get(n);
                }
                
                Punkt pa = (Punkt)obwiednia.get(n-1);
                
                a = (double)(pa.getY() - pb.getY())/(double)(pa.getX()-pb.getX());
                b = (double)(pb.getY() - a * pb.getX());
                
                pitch[i] = (int)Math.round((double)(mi.getPoczatek(k)-i)/(double)(mi.getPoczatek(k)-mi.getKoniec(k))*a+b);
                
            }
        }
        
        // akcenty
        
        int poczatek = 0;
        int koniec = 0;
        
        while(poczatek<w.size()) {
            
            koniec = poczatek + 1;
            while(koniec<w.size() && !((String)w.get(koniec)).equals("_")) {
                koniec++;
            }
            
            if(koniec - poczatek>3) {
                
                double len = koniec - poczatek;
                
                for(int i=poczatek; i<=koniec; i++) {
                    
                    if(
                            (i-poczatek) >= (int)Math.round(len*0.4) &&
                            (i-poczatek) < (int)Math.round(len*0.48)) {
                        
                        //pitch[i] = (int)Math.round((double)podstawaPitch*(1.05+Math.random()*0.075));
                        //pitch[i] = (int)Math.round(podstawaPitch-(double)podstawaPitch*(.05+Math.random()*0.075));
                        
                        pitch[i] = (int)Math.round((double)pitch[i]*0.97);
                        
                    } else if(
                            (i-poczatek) >= (int)Math.round(len*0.48) &&
                            (i-poczatek) <= (int)Math.round(len*0.65)) {
                        
                        //pitch[i] = (int)Math.round((double)podstawaPitch*(1.1+Math.random()*0.1));
                        //pitch[i] = (int)Math.round(podstawaPitch-(double)podstawaPitch*(.1+Math.random()*0.1));
                        
                        pitch[i] = (int)Math.round((double)pitch[i]*0.94);
                        
                        czas[i] = (int)Math.round((double)czas[i]*1.075);
                        
                    } else if(
                            (i-poczatek) > (int)Math.round(len*0.65) &&
                            (i-poczatek) < (int)Math.round(len*0.75)) {
                        
                        //pitch[i] = (int)Math.round((double)podstawaPitch*(1.025+Math.random()*0.8));
                        //pitch[i] = (int)Math.round(podstawaPitch-(double)podstawaPitch*(.025+Math.random()*0.08));
                        
                        pitch[i] = (int)Math.round((double)pitch[i]*0.97);
                        
                    }
                }
            }
            
            czas[koniec-1] = (int)Math.round((double)czas[koniec-1]*1.085);
            
            if(koniec<w.size())
                czas[koniec] = (int)Math.round((double)czas[koniec]*1.125);
            
            poczatek = koniec;
        }
        
    }
    
    public void show() {
        JFrame f = new JFrame(this.getString());
        f.getContentPane().add(new TranskrybowanePanel(this));
        f.setSize(500, 400);
        f.setVisible(true);
    }
    
    public void write() {
        
        Writer bf;
        
        try {
            bf = new BufferedWriter(new FileWriter("sampa_output.pho"));
            
            for(int i=0; i<w.size(); i++) {
                
                String s = (String)w.get(i);
                
                if(!s.equals("!") && !s.equals("?") && !s.equals(".")) {
                    bf.write(s);
                    bf.write(" "+czas[i]+" 0 "+pitch[i]+"\n");
                } else {
                    bf.write("_");
                    bf.write(" "+czas[i]+" 0 "+pitch[i]+"\n");
                }
                
            }
            
            bf.flush();
            bf.close();
            
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
    
    public String getString() {
        String zwracany = "";
        
        for(int i=0; i<w.size(); i++) {
            
            String s = (String)w.get(i);
            
            if(!s.equals("!") && !s.equals("?") && !s.equals(".")) {
                zwracany += (String)w.get(i);
            } else {
                zwracany += "_";
            }
                    
        }
        return zwracany;
    }
    
}
